#include "stdafx.h"
#include "LightingDemo.h"

void LightingDemo::Initialize()
{
	Context::Get()->GetCamera()->RotationDegree(20, 0, 0);
	Context::Get()->GetCamera()->Position(1, 36, -85);
	((Freedom*)Context::Get()->GetCamera())->Speed(50);

	shader = new Shader(L"36_Material.fxo");
	sky = new CubeSky(L"Environment/GrassCube1024.dds");

	Mesh();
	Kachujin();
	Airplane();
	Weapon();
}

void LightingDemo::Destroy()
{
	SafeDelete(shader);
	//SafeDelete(LightShader);
	SafeDelete(sky);

	//Material
	SafeDelete(floor);
	SafeDelete(stone);
	SafeDelete(brick);
	SafeDelete(wall);

	//Mesh
	SafeDelete(cube);
	SafeDelete(plane);
	SafeDelete(cylinder);
	SafeDelete(sphere);

	//Model + anim
	SafeDelete(airplane);
	SafeDelete(kachujin);
	SafeDelete(weapon);

}

void LightingDemo::Update()
{
	ImGui::SliderFloat3("LightDirection", Context::Get()->Direction(), -1, 1);
	//ImGui::ColorEdit3("floor SpecularColor", floor->Specular());
	//ImGui::SliderFloat("floor Specular Intensity", &floor->Specular().a , 0.1f , 30.0f);
	//ImGui::ColorEdit3("Birck SpecularColor", brick->Specular());
	//ImGui::SliderFloat("Birck Specular Intensity", &brick->Specular().a, 0.1f, 30.0f);
	
	ImGui::ColorEdit3("Birck Emissive", brick->Emissive());
	ImGui::SliderFloat("Birck Emissive Intensity", &brick->Emissive().a, 0.0f, 2);



	sky->Update();

	cube->Update();
	plane->Update();
	cylinder->Update();
	sphere->Update();
	
	airplane->Update();
	kachujin->Update();
	
	// 
	Matrix worlds[MAX_MODEL_TRANSFORMS];
	for (UINT i = 0; i < kachujin->GetTrasnformCount(); i++)
	{
		kachujin->GetAttachBones(i, worlds);
		weapon->GetTransform(i)->World(weaponTransform->World() * worlds[40]);
	}

	// 
	static Color swordColor = Color(0, 0, 0, 1);
	ImGui::ColorEdit3("SwordColor", (float*)&swordColor);
	weapon->SetColor(1, swordColor);

	weapon->UpdateTransforms();
	weapon->Update();
	
}

void LightingDemo::Render()
{
	//Background
	sky->Render();
	Pass(0);

	//Mesh
	{
		wall->Render();
		sphere->Render();

		brick->Render();
		cylinder->Render();

		stone->Render();
		cube->Render();

		floor->Render();
		plane->Render();
	}

	airplane->Render();
	kachujin->Render();
	weapon->Render();
}

void LightingDemo::Mesh()
{
	//CreateMaterials
	{
		floor = new Material(shader);
		floor->DiffuseMap("Floor.png");
		floor->Specular(1, 1, 1, 20);
		floor->SpecularMap(L"Floor_Specular.png");

		stone = new Material(shader);
		stone->DiffuseMap("Stones.png");
		stone->Specular(1, 1, 1, 20);
		stone->SpecularMap(L"Stones_Specular.png");
		stone->Emissive(0.15f, 0.15f, 0.15f, 0.3f);

		brick = new Material(shader);
		brick->DiffuseMap("Bricks.png");
		brick->Specular(0.5, 0.5, 1, 50);
		brick->SpecularMap(L"Bricks_Specular.png");
		brick->Emissive(0.15f, 0.15f, 0.15f, 0.3f);

		wall = new Material(shader);
		wall->DiffuseMap("Wall.png");
		wall->Specular(1, 1, 1, 20);
		wall->SpecularMap(L"Wall_Specular.png");
		wall->Emissive(0.15f, 0.15f, 0.15f, 0.3f);
	}

	//CreateMesh
	{
		Transform* transform = NULL;
		cube = new MeshRender(shader, new MeshCube());
		transform = cube->AddTransform();
		transform->Scale(20, 10, 20);
		transform->Position(0, 5, 0);

		plane = new MeshRender(shader, new MeshPlane(2.5, 2.5));
		transform = plane->AddTransform();
		transform->Scale(12, 1, 12);

		cylinder = new MeshRender(shader, new MeshCylinder(0.3f, 0.5f, 3.0f, 20, 20));
		sphere = new MeshRender(shader, new MeshSphere(0.5f, 20, 20));

		for (UINT i = 0; i < 5; i++)
		{
			transform = cylinder->AddTransform();
			transform->Position(-30, 6, (float)i * 15.0f - 15.0f);
			transform->Scale(5, 5, 5);

			transform = cylinder->AddTransform();
			transform->Position(+30, 6, (float)i * 15.0f - 15.0f);
			transform->Scale(5, 5, 5);

			transform = sphere->AddTransform();
			transform->Position(-30, 15.5f, (float)i * 15.0f - 15.0f);
			transform->Scale(5, 5, 5);

			transform = sphere->AddTransform();
			transform->Position(+30, 15.5f, (float)i * 15.0f - 15.0f);
			transform->Scale(5, 5, 5);

		}
	}

	//
	cube->UpdateTransforms();
	plane->UpdateTransforms();
	cylinder->UpdateTransforms();
	sphere->UpdateTransforms();

	//  vector
	meshes.push_back(cube);
	meshes.push_back(plane);
	meshes.push_back(cylinder);
	meshes.push_back(sphere);
}

void LightingDemo::Airplane()
{
	airplane = new ModelRender(shader);
	airplane->ReadMesh(L"B787/Airplane");
	airplane->ReadMaterial(L"B787/Airplane");

	Transform* transform = airplane->AddTransform();
	transform->Position(Vector3(2.0f, 9.90f, 2.0f));
	transform->Scale(0.004f, 0.004f, 0.004f);
	airplane->UpdateTransforms();

	models.push_back(airplane);
}

void LightingDemo::Kachujin()
{
	kachujin = new ModelAnimator(shader);
	kachujin->ReadMesh(L"Kachujin/Mesh");
	kachujin->ReadMaterial(L"Kachujin/Mesh");

	kachujin->ReadClip(L"Kachujin/Idle");
	kachujin->ReadClip(L"Kachujin/Walk");
	kachujin->ReadClip(L"Kachujin/Run");
	kachujin->ReadClip(L"Kachujin/Slash");
	kachujin->ReadClip(L"Kachujin/Uprock");

	Transform* transform = NULL;

	transform = kachujin->AddTransform();
	transform->Position(0, 0, -30);
	transform->Scale(0.075f, 0.075f, 0.075f);
	kachujin->PlayTweenMode(0, 0, 1.0f);

	transform = kachujin->AddTransform();
	transform->Position(-15, 0, -30);
	transform->Scale(0.075f, 0.075f, 0.075f);
	kachujin->PlayTweenMode(1, 1, 1.0f);
	
	transform = kachujin->AddTransform();
	transform->Position(-30, 0, -30);
	transform->Scale(0.075f, 0.075f, 0.075f);
	kachujin->PlayTweenMode(2, 2, 1.0f);
	
	transform = kachujin->AddTransform();
	transform->Position(15, 0, -30);
	transform->Scale(0.075f, 0.075f, 0.075f);
	kachujin->PlayBlendMode(3, 0, 1, 2);
	kachujin->SetBlendAlpha(3, 1.75f);

	transform = kachujin->AddTransform();
	transform->Position(30, 0, -30);
	transform->Scale(0.075f, 0.075f, 0.075f);
	kachujin->PlayTweenMode(4, 4, 1.0f);

	kachujin->UpdateTransforms();
	animators.push_back(kachujin);
}

void LightingDemo::Weapon()
{
	weapon = new ModelRender(shader);
	weapon->ReadMesh(L"Weapon/Sword");
	weapon->ReadMaterial(L"Weapon/Sword");

	UINT count = kachujin->GetTrasnformCount();
	for (UINT i = 0; i < count; i++)
		weapon->AddTransform();

	weapon->UpdateTransforms();
	models.push_back(weapon);

	weaponTransform = new Transform();
	weaponTransform->Position(-2.9f, 1.45f, -6.45f);
	weaponTransform->Scale(0.5f, 0.5f, 0.75f);
	weaponTransform->Rotation(0, 0, 1);
}

void LightingDemo::Pass(UINT val)
{
	for (MeshRender* mesh : meshes)
		mesh->Pass(val);
	
	val++;
	for (ModelRender* model : models)
		model->Pass(val);
	
	val++;
	for (ModelAnimator* anim : animators)
		anim->Pass(val);
}

